// Javascript functions used by all of the html files.
// These are loaded by the line
// <script language="javascript" src="functions.js"></script>
// in the header.

var gCssSize = "cssSize";
var gSaveLocation = "SaveLocation";
var gSaveState = "SaveState";

//--------------------------------------------------------//
// Cursor Hiding Functions
//
// These functions assume you have an empty cursor called 
// invisible.cur in your images directory.
//
function HideCursorEverywhere()
{
    //document.body.style.cursor = "url('images/invisible.cur'),default";
    document.body.style.cursor = "none";
}

function HideCursorInImage()
{
    document.getElementById("image").style.cursor = "none";
}

function ShowCursorEverywhere()
{
    document.body.style.cursor = "";
}

function ShowCursorInImage()
{
    document.getElementById("image").style.cursor = "";
}


//--------------------------------------------------------//
// MouseData Object
//
// An object used for dealing with Mouse Data.
//
// The startSide is the side of the image the mouse starts on
// 123
// 456
// 789
// If 4 or 6, y data is ignored.  If 2 or 8, x data is ignored.
// When points come in that are closer to the startSide than 
// the initial point, the initial point is updated.  This doesn't
// happen with the middle startSide (5).
function MouseData(startSide)
{
    this.x1 = 0;
    this.x2 = 0;
    this.y1 = 0;
    this.y2 = 0;
    this.direction = 0;
    this.distance = 0;
    this.ignoreX = false;
    this.ignoreY = false;
    this.firstPoint = true;
    this.startSide = startSide;

    this.SetPoint = SetPoint;
    this.SetPoint1 = SetPoint1;
    this.SetPoint2 = SetPoint2;
    this.GetDistance = GetDistance;
    this.SetStartSide = SetStartSide;
    this.MoveStartPointIfNecessary = MoveStartPointIfNecessary;

    this.SetStartSide(startSide);
}

// This is the function used to feed mouse data to the object.
function SetPoint(x, y)
{
    if (this.firstPoint)
    {
        this.SetPoint1(x, y);
        this.SetPoint2(x, y);
        this.firstPoint = false;
    }
    else
    {
        this.SetPoint2(x, y);
    }
}

// This is the function used to see how far the mouse has
// moved from the initial point.
function GetDistance()
{
    var xDist = this.x1 - this.x2;
    var yDist = this.y1 - this.y2;
    return Math.sqrt(xDist * xDist + yDist * yDist);
}


// The rest of these functions are internal to the object and 
// probably don't need to be called directly
function SetStartSide(startSide)
{
    if (startSide < 1 || startSide > 9)
    {
        startSide = 5;
    }

    this.startSide = startSide;

    this.ignoreX = false;
    this.ignoreY = false;

    switch (startSide)
    {
        case 2:
        case 8:
            this.ignoreX = true;
            break;

        case 4:
        case 6:
            this.ignoreY = true;
            break;
    }
}

function SetPoint1(x1, y1)
{
    if (!this.ignoreX)
    {
        this.x1 = x1;
    }

    if (!this.ignoreY)
    {
        this.y1 = y1;
    }
}

function SetPoint2(x2, y2)
{
    if (!this.ignoreX)
    {
        this.x2 = x2;
    }

    if (!this.ignoreY)
    {
        this.y2 = y2;
    }

    this.MoveStartPointIfNecessary();
}

function MoveStartPointIfNecessary()
{
    var side = this.startSide;
    if (side == 5)
    {
        return;
    }

    if ((side <= 3 && this.y2 < this.y1) || (side >= 7 && this.y2 > this.y1))
    {
        this.y1 = this.y2;
    }

    var mod = side % 3;

    if ((mod == 1 && this.x2 < this.x1) || (mod == 0 && this.x2 > this.x1))
    {
        this.x1 = this.x2;
    }
}


//--------------------------------------------------------//
// AnimHelper Object
//
// An object used to update the images in an animation based
// on mouse data. It assumes the existence of an 
// AdventureCreator anim object. It also assumes that all of 
// the images in the animation are the same size.
//
// The startSide is the side of the image the mouse starts on
// 123
// 456
// 789
// If 4 or 6, y data is ignored.  If 2 or 8, x data is ignored.
// When points come in that are closer to the startSide than 
// the initial point, the initial point is updated.  This doesn't
// happen with the middle startSide (5).

function AnimHelper(startSide)
{
    this.numPicts = anim.pictures.length;
    this.maxPict = this.numPicts - 1;
    this.increment = 1
    this.startSide = startSide;

    this.fasterThanIn = 0;
    this.slowerThanIn = 0;
    this.fasterThanOut = 0;
    this.slowerThanOut = 0;
    this.inDelay = 0;
    this.outDelay = 0;
    this.failed = false;

    this.startTimeIn = new Date();
    this.startTimeOut = new Date();
    this.startTimeInDelay = new Date();
    this.startTimeOutDelay = new Date();

    this.maxOverride = this.maxPict;
    this.maxOverrideAutoIncrementAmount = 0;
    this.maxOverrideAutoIncrementFrame = 0;
    this.maxOverrideCanIncrement = false;

    this.prevIndex = 0;

    this.mouseData = new MouseData(startSide);

    this.UpdateImage = UpdateImage;
    this.SetTimes = SetTimes;
    this.SetMaxOverride = SetMaxOverride;
    this.AutomateMaxOverride = AutomateMaxOverride;
    this.RestartTiming = RestartTiming;

    this.SetIncrement = SetIncrement;
    this.UpdateFailedState = UpdateFailedState;

    this.SetIncrement();
}

function UpdateImage(event)
{
    var imageChanged = false;

    var image = anim.pictures[0];
    var mouseX = event.clientX - image.offsetLeft;
    var mouseY = event.clientY - image.offsetTop;

    this.mouseData.SetPoint(mouseX, mouseY);

    var dist = this.mouseData.GetDistance();
    var index = Math.floor(dist / this.increment);

    // The math says this isn't necessary, but it's better to be safe than sorry.
    if (index > this.maxPict)
    {
        index = this.maxPict;
    }
    else if (index < 0)
    {
        index = 0;
    }


    if (index == this.maxOverrideAutoIncrementFrame && this.maxOverrideAutoIncrementAmount != 0 && this.maxOverrideCanIncrement)
    {
        this.SetMaxOverride(this.maxOverride + this.maxOverrideAutoIncrementAmount);
        this.maxOverrideCanIncrement = false;
    }
    else if (index == this.maxOverride && this.maxOverrideAutoIncrementAmount != 0)
    {
        this.maxOverrideCanIncrement = true;
    }
    else if (index > this.maxOverride)
    {
        this.failed = true;
        index = this.maxOverride;
        // The image didn't actually change
        // but the caller needs to know 
        // to check the failed state
        imageChanged = true;
    }

    if (index != anim.curPicture)
    {
        imageChanged = true;

        this.prevIndex = anim.curPicture;
        this.UpdateFailedState(index, this.prevIndex);

        anim.curPicture = index;
        document.getElementById("image").src = anim.pictures[anim.curPicture].src;
    }

    return imageChanged;
}


// The MaxOverride limits the number of frames that can play.
// For instance, if there are 10 pictures in the animation
// but you only want the user to be able to go halfway, 
// set the MaxOverride to 5.  
// To clear the override, set it to -1.
function SetMaxOverride(override)
{
    if (override < 0 || override > this.maxPict)
    {
        override = this.maxPict;
    }

    this.maxOverride = override;
}

// This will automatically add incrementAmount to the MaxOverride
// every time the frame gets to the MaxOverride and then goes
// back to incrementFrame.  To disable this functionality set
// the incrementAmount to 0.
function AutomateMaxOverride(incrementAmount, incrementFrame)
{
    this.maxOverrideAutoIncrementAmount = incrementAmount;
    this.maxOverrideAutoIncrementFrame = incrementFrame;
}


// Times are in Milliseconds.  0 means that timeout is not used.
// "In" is the time to go from frame 0 to maxPict.
// "Out" is the time to go from maxPict to 0.
// "FasterThan" means it has to take less time than that.
// "SlowerThan" means it has to take more time than that.
// "inDelay" is how long it has to stay at frame 0.
// "outDelay" is how long it has to stay at frame maxPict.
function SetTimes(fasterThanIn, slowerThanIn, inDelay, fasterThanOut, slowerThanOut, outDelay)
{
    this.fasterThanIn = fasterThanIn;
    this.slowerThanIn = slowerThanIn;
    this.inDelay = inDelay;
    this.fasterThanOut = fasterThanOut;
    this.slowerThanOut = slowerThanOut;
    this.outDelay = outDelay;
}

// Times normally start when the page loads.
// Call this function if you want to restart them.
function RestartTiming()
{
    this.startTimeIn = new Date();
    this.startTimeOut = new Date();
    this.startTimeInDelay = new Date();
    this.startTimeOutDelay = new Date();
}


// The rest of these functions are internal to the object and 
// probably don't need to be called directly
function UpdateFailedState(cur, prev)
{
    var curTime = new Date();
    var elapsedIn = curTime - this.startTimeIn;
    var elapsedOut = curTime - this.startTimeOut;
    var elapsedInDelay = curTime - this.startTimeInDelay;
    var elapsedOutDelay = curTime - this.startTimeOutDelay;
    var failed = false;

    var max = this.maxPict;
    var pent = this.maxPict - 1;

    if (cur == 1 && prev == 0)
    {
        if (this.inDelay != 0 && elapsedInDelay < this.inDelay)
        {
            failed = true;
        }

        this.startTimeIn = curTime;
    }
    else if (cur == 0 && prev == 1)
    {
        if ((this.fasterThanOut != 0 && elapsedOut > this.fasterThanOut) ||
            (this.slowerThanOut != 0 && elapsedOut < this.slowerThanOut))
        {
            failed = true;
        }

        this.startTimeInDelay = curTime;
    }
    else if (cur == max && prev == pent)
    {
        if ((this.fasterThanIn != 0 && elapsedIn > this.fasterThanIn) ||
            (this.slowerThanIn != 0 && elapsedIn < this.slowerThanIn))
        {
            failed = true;
        }

        this.startTimeOutDelay = curTime;
    }
    else if (cur == pent && prev == max)
    {
        if (this.outDelay != 0 && elapsedOutDelay < this.outDelay)
        {
            failed = true;
        }

        this.startTimeOut = curTime;
    }

    if (failed)
    {
        this.failed = true;
    }
}


function SetIncrement()
{
    var image = anim.pictures[0];
    var width = image.width;
    var height = image.height;

    var side = this.startSide;

    if (side == 4 || side == 6) // Horizontal movement
    {
        this.increment = width / this.numPicts;
    }
    else if (side == 2 || side == 8) // Vertical movement
    {
        this.increment = height / this.numPicts;
    }
    else // Diagonal movement
    {
        this.increment = Math.sqrt(width * width + height * height) / this.numPicts;
    }
}



//--------------------------------------------------------//
// Animation timer
//
// This function is called for animations.  It shouldn't
// need to be called directly
function Timer(anim)
{
    anim.curPicture++;
    if (anim.curPicture == anim.pictures.length)
    {
        if (anim.loop)
        {
            anim.curPicture = 0
        }
        else
        {
            anim.curPicture--;
            clearInterval(anim.timerId);
        }
    }

    document.getElementById("image").src = anim.pictures[anim.curPicture].src;
}

//--------------------------------------------------------//
// Sound playing function
//
// This funcion plays a sound that was added through 
// AdventureCreator.  The id is "sound0" for the first
// sound, "sound1" for the second, etc.
function PlaySound(id)
{
    document.getElementById(id).play();
}



//--------------------------------------------------------//
// Function to update the toptext
//
// This function changes the text above the picture. 
// If clear is true, it replaces the top text with str.
// If clear is false, it adds str to the top text.
function UpdateTopText(str, clear)
{
    var element = document.getElementById("toptext");
    if (clear)
    {
        element.innerHTML = str;
    }
    else
    {
        element.innerHTML += str;
    }
}



function PickSizeCss()
{
    var cssFile = "_size.css";
    var size = readVar(gCssSize);

    if (size > 0)
    {
        cssFile = "_size2.css";
    }

    document.write("<link href=\"" + cssFile + "\" rel=\"stylesheet\" type=\"text/css\" />");
}


function ChooseSize()
{
    var size = 0;

    if (typeof (gameLargeScreenWidth) != "undefined" && typeof (gameLargeScreenHeight) != "undefined")
    {
        if (screen.width >= gameLargeScreenWidth && screen.height >= gameLargeScreenHeight)
        {
            size = 1;
        }
    }

    setVar(gCssSize, size);
    return size;
}



//<area shape="rect" coords="80,43,552,405" onclick="return foo();" href="intro1.html" title="Goto 1" />

function AddArea(x1, y1, x2, y2, onclick, href, title)
{
    var size = readVar(gCssSize);
    var str;

    if (size > 0 && typeof(gameScaleFactor) != "undefined")
    {
        x1 *= gameScaleFactor;
        y1 *= gameScaleFactor;
        x2 *= gameScaleFactor;
        y2 *= gameScaleFactor;
    }

    str = "<area shape=\"rect\" coords=\"" + x1 + "," + y1 + "," + x2 + "," + y2 + "\" ";
    
    if (onclick != "#")
    {
        str += "onclick=\"" + onclick + "\" ";
    }

    str += "href=\"" + href + "\" title=\"" + title + "\" />";

    document.write(str);
}


function CheckIfSaveExists()
{
    var saveData = parseCookie(gSaveState);
    if (saveData['SaveLocation'] === undefined)
        return false;
    else
        return true;
}


function SaveState(page)
{
    if (typeof (page) == "undefined")
        page = document.URL;
    var saveData = {'SaveLocation' : page};

    for (var index in gameVars)
    {
        var from = gameVars[index];
        saveData[from] = readVar(from);
    }

    dumpCookie(gSaveState, saveData);

    return false;
}


function RestoreState()
{
    var saveData = parseCookie(gSaveState);

    for (var index in gameVars)
    {
        var to = gameVars[index];
        if (saveData[to] === undefined)
            setVar(to, 0);
        else
            setVar(to, saveData[to]);
    }

    window.location = saveData['SaveLocation'];

    return false;
}



// This function can replace simple "check" files. Give it the variable and a check value, and it 
// will navigate to the link1 if the variable is greater than the check.  It wil
// navigate to link2 if the variable is less than or equal to the check.
// ie:  Choose('cinema', 0, 'beenthere.html', 'cinema1.html');
function Choose(name, check, link1, link2)
{
    var val = readVar(name);
    if (val > check)
    {
        window.location = link1;
    }
    else
    {
        window.location = link2;
    }

    return false;
}



//--------------------------------------------------------//
// Cookie (variable) functions
//
// These functions are used for storing and changing 
// variables in the site.


// Loop through available cookies and return data matching name
function readCookie(name)
{
    var fullName = name + "=";
    var val = "";
    var strings = document.cookie.split(';');
    for (var i = 0; i < strings.length; i++)
    {
        var str = strings[i];
        while (str.charAt(0) == ' ')
            str = str.substring(1, str.length);
        if (str.indexOf(fullName) == 0)
        {
            val = str.substring(fullName.length, str.length)
            break;
        }
    }
    return unescape(val);
}

// Write data to named cookie, creating if necessary
function writeCookie(name, value, expires)
{
    if (expires == null)
    {
        var days = 30; // Number of days to keep the cookies
        expires = new Date();
        expires.setTime(expires.getTime + (days * 24 * 60 * 60 * 1000));
    }

    document.cookie = name + "=" + escape(value) + "; path=/" + ((expires == null) ? "" : "; expires=" +expires.toGMTString());
}

// Delete existing cookie
function deleteCookie(name)
{
    document.cookie = name + "=; expires=Thu, 01-Jan-70 00:00:01 GMT" + "; path=/";
}

// Read JSON data from cookie, and parse into data object
function parseCookie(name)
{
    var val = readCookie(name);
    if (val == "")
        return new Object;

    // decode Base64 to normal ASCII JSON
    val = Base64.decode(val);

    // parse JSON into object
    val = JSON.parse(val);

    return val; 
}

// Write object to cookie as Base64-encoded JSON
function dumpCookie(name, data)
{
    // serialize data to JSON string
    data = JSON.stringify(data);

    // encode data to Base64 for character safety
    data = Base64.encode(data);

    writeCookie(name, data);
}

// This function deletes a variable from the primary cookie
function deleteVar(name)
{
    var data = parseCookie('workingset');
    delete data[name];
    dumpCookie('workingset', data);
}

// This function sets a numerical variable and rewrites the primary cookie
function setVar(name, value)
{
    var data = parseCookie('workingset');
    data[name] = value;
    dumpCookie('workingset', data);
}

// This function extracts one numerical variable from the primary cookie
function readVar(name)
{
    var data = parseCookie('workingset');
    var val = data[name];

    // check if variable was ever set
    if (val === undefined)
        val = 0;

    // check if NaN
    else if (isNaN(val))
        val = 0;

    return val;
}

// This function sets a string variable and rewrites the primary cookie
function setString(name, value)
{
    var data = parseCookie('workingset');
    data[name] = value;
    dumpCookit('workingset', data);
}

// This function extracts one string variable from the primary cookie
function readString(name)
{
    var data =  parseCookie('workingset');
    var val = data[name];

    // check if variable was ever set
    if (val === undefined)
        val = "";

    return val;
}

function initVars()
{
    for (var val in gameVars)
    {
        setVar(gameVars[val], 0);
    }

    ChooseSize();
}

// If the variable's value is 0, this sets it to 1.
// If the variable's value is anything else, this sets it to 0.
function varToggle(name)
{
    var val = readVar(name);
    if (val == 0)
    {
        val = 1;
    }
    else
    {
        val = 0;
    }
    setVar(name, val);
}

// This function adds 1 to a variable's value
function varPlus1(name)
{
    var val = readVar(name);
    val += 1;
    setVar(name, val);
}

// This function adds 1 to a variable's value if the second variable is 0
function varPlus1If0(name, checkName)
{
    var val = readVar(name);
    var check = readVar(checkName);
    if (check == 0)
    {
        val += 1;
        setVar(name, val);
    }
}

// This function adds an arbitrary amount to the variable's value
function varPlus(name, amount)
{
    var val = readVar(name);
    val += amount;
    setVar(name, val);
}


// This function subtracts 1 from a variable's value. 
// If the value would have gone negative, it sets the value to 0.
function varMinus1(name)
{
    var val = readVar(name);
    if (val > 1)
        val -= 1;
    else
        val = 0;        
    setVar(name, val);
}

// This function subtracts an arbitrary amount from a variable's value. 
// If the value would have gone negative, it sets the value to 0.
function varMinus(name, amount)
{
    var val = readVar(name);
    if (val > amount)
        val -= amount;
    else
        val = 0;        
    setVar(name, val);
}


// The next three functions are "bitwise."  They are used for 
// packing multiple "yes/no" variables into one cookie.
// The index starts from 0.
function varSetBit(name, index)
{
    var val = readVar(name);
    var mask = 1 << index;
    val = val | mask;
    setVar(name, val);
}

function varClearBit(name, index)
{
    var val = readVar(name);
    var mask = ~(1 << index);
    val = val & mask;
    setVar(name, val);
}

function varReadBit(name, index)
{
    var val = readVar(name);
    val = (val >> index) & 0x01;
    return val;
}

function varCountBits(name)
{
    var val = readVar(name);
    var count = 0;
    while (val != 0)
    {
        count += val & 0x01;
        val >>= 1;
    }

    return count;
}

// This is a debugging function.  Use it to print the value of a variable 
//on the screen.
function outputVar(name)
{
    var val = readVar(name);
    //document.write("<p>" + name + " = " + val + "</p>");  

    document.write("<p>" + name + " = " + val + " <a onclick=\"varMinus1('" + name + "');\" href='#'>sub</a> <a onclick=\"varPlus1('" + name + "');\" href='#'>add</a></p>");    
}

// This is a function for outputing a string centered on the screen.  
//It is typically used for displaying the score.
function outputString(string)
{
    document.write("<div align='center'><p>" + string + "</p></div>");
}


//--------------------------------------------------------//
// Navigation disabling functions
// 
// These functions are used for disabling various forms
// of navigation on the site (especially pressing the
// back button).  


// This is the entry point for all navigation disabling functions.
// It is called from each page's body onload.
function disallowNav()
{

    var allowNav = 0;
    
    if (typeof (gameAllowNav) != "undefined")
    {
        allowNav = gameAllowNav;
    }

    if (!allowNav)
    {
        document.oncontextmenu = blockContextMenu;
        document.onkeydown = blockKeyPresses;

        // This part works for IE, but not Firefox
        undoBack();
        // This extra part is need for Firefox
        window.onload = window.onload; undoBack();
        window.onpageshow = function (evt) { if (evt.persisted) undoBack() };
        window.onunload = function () { void (0) };
    }
}


// If the user goes back, drive him forward again
function undoBack()
{
    window.history.forward();
}

// Disallow the context menu
function blockContextMenu()
{
	event.cancelBubble = true
   	event.returnValue = false;
  	return false;
}

// This is kind of overkill, but it will block the user
// from hitting backspace or F5
function blockKeyPresses() 
{
    var keyBackspace = 8;
    var keyF5 = 116;
    
	if ((event.keyCode == keyBackspace) || (event.keyCode == keyF5)) 
   	{
   	    event.keyCode = 0;
   	    event.returnValue = false;
   	}
}



//--------------------------------------------------------//
// Debug Function
// 
// This function is used for debugging the game.
// To make use of it, go into your _game.js file and set
// "gameDoDebug" to 1. You will also have needed to set your 
// game variables into an array called gameVars.
// For example, the top lines of your _game.js would look like this:
//var gameVars = ["happy", "angry", "difficulty"];
//var gameDoDebug = 1;

function debug()
{
    var doDebug = 0;

    if (typeof (gameDoDebug) != "undefined")
    {
        doDebug = gameDoDebug;
    }

    if (doDebug)
    {
        for (var val in gameVars)
        {
            outputVar(gameVars[val]);
        }
    }
}

//--------------------------------------------------------//
// ShowScore Function
// 
// This function is used for showing the score in the game.
// It will only work if you have a gameShowScore() function 
// in your _game.js file. 

function showScore()
{
    if (typeof (gameShowScore) != "undefined")
    {
        gameShowScore();
    }
}
