// _game.js is a set of functions that changes from game to game.
// Anything game-specific should go in this file.  Examples include
// the definition of any game variables, and the check functions.


// This is the list of variables you use in your game.
var gameVars = ["happy", "angry", "difficulty", "haveKey", "cssSize"];

// Set this to 1 to see debugging information when testing your game
// It should be set to 0 when people are playing the game.
var gameDoDebug = 1;

// Set this to 1 to allow using the back button while testing your game
// It should be set to 0 when people are playing the game.
var gameAllowNav = 1;

// To use the larger image size, this is the minimum size the users monitor must be
var gameLargeScreenWidth = 1280;
var gameLargeScreenHeight = 900;

// when using the larger image size, this is the amount everything is scaled by
var gameScaleFactor = 1.25;


// These are constants used by the game.  These are used by the "calcNeeded" function below.
var moveEasy = 1;
var moveMedium = 2;
var moveHard = 3;


// If your game shows the current score at the bottom, you need 
// this function.  You can change the wording any way you'd like.
function gameShowScore()
{
    var score = calcScore();
    var needed = calcNeeded("move");
    var str;

    str = "Your current score is " + score + ". You need " + needed + " to open the door.";

    var save = " <a onclick=\"SaveState();\" href='#'>Save</a> ";
    var restore = " <a onclick=\"RestoreState();\" href='#'>Restore</a> ";

    outputString(str + save + restore);
}


// The current score is some combination of variables.  In this simple example
// it's just happy - angry.  You might do something more complicated in your game
function calcScore()
{
    var happy = readVar("happy") - readVar("angry");
    return happy;
}

// This is a calculation of what's needed to continue.  It gathers the requirement
// from constants defined in _begin.html.  For instance, this game has 
// constants for moveEasy, moveMedium, and moveHard.  When using those constants
// you send "move" to this function.  You can have other constants for other 
// areas of the game.  Just start them all with the same word and end them in 
// Easy, Medium, and Hard.  (ie dateEasy, dateMedium, and dateHard)
function calcNeeded(area)
{
    var needed = 0;
    var difficulty = readVar("difficulty");

    // If the area passed in is "move" then easy will be "moveEasy".
    // doorEasy is one of the constants set in _begin.html
    var easy = area.concat("Easy");
    var medium = area.concat("Medium");
    var hard = area.concat("Hard");

    switch (difficulty)
    {
        case 1: // Medium
            needed = window[medium];
            break;

        case 2: // Hard
            needed = window[hard];
            break;

        default: // Easy
            needed = window[easy];
            break;
    }

    return needed;
}


// This function checks to see if your score is sufficient to get 
// Tlaero to move.  It also checks to see if you already have the 
// key or not, and sends you to the appropriate room.
// Note that the "return false" is necessary at the bottom.  That
// works around a bug in Firefox.
function checkMove()
{
    var score = calcScore();
    var needed = calcNeeded("move");

    var key = readVar("haveKey");

    if (score >= needed)
    {
        if (key > 0)
        {
            window.location = "keyroom2.htm";
        }
        else
        {
            window.location = "keyroom1.htm";
        }
    }
    else
    {
        window.location = "game2.htm";
    }

    return false;
}


// This simple function checks if you've got the key and
// sends you to the right place.
// Note that the "return false" is necessary at the bottom.  That
// works around a bug in Firefox.
function checkKey()
{
    var haveKey = readVar("haveKey");

    if (haveKey > 0)
    {
        window.location = "finish.htm";
    }
    else
    {
        window.location = "locked.htm";
    }

    return false;
}

// This simple function checks if you're playing at the appropriate
// difficulty and sends you to the right place.
// Note that the "return false" is necessary at the bottom.  That
// works around a bug in Firefox.
function checkEasy()
{
    var difficulty = readVar("difficulty");

    if (difficulty > 0)
    {
        window.location = "continuemedium.htm";
    }
    else
    {
        window.location = "finisheasy.htm";
    }

    return false;
}


// This simple function checks if you're playing at the appropriate
// difficulty and sends you to the right place.
// Note that the "return false" is necessary at the bottom.  That
// works around a bug in Firefox.
function checkMedium()
{
    var difficulty = readVar("difficulty");

    if (difficulty > 1)
    {
        window.location = "finishhard.htm";
    }
    else
    {
        window.location = "finishmedium.htm";
    }

    return false;
}